@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: setup.c 1.12 2011/05/15 14:47:45 kls Exp $
+ * $Id: setup.c 1.14 2011/12/04 15:31:58 kls Exp $
  */
 
 #include "setup.h"
@@ -20,11 +20,12 @@ cHdffSetup gHdffSetup;
 cHdffSetup::cHdffSetup(void)
 {
     Resolution = kResolution1080i;
-    TvFormat = HDFF::tvFormat16by9;
-    VideoConversion = HDFF::videoConversionPillarbox;
-    AnalogueVideo = HDFF::videoOutCvbsYuv;
+    VideoModeAdaption = HDFF_VIDEO_MODE_ADAPT_OFF;
+    TvFormat = HDFF_TV_FORMAT_16_BY_9;
+    VideoConversion = HDFF_VIDEO_CONVERSION_PILLARBOX;
+    AnalogueVideo = HDFF_VIDEO_OUT_CVBS_YUV;
     AudioDelay = 0;
-    AudioDownmix = HDFF::downmixAutomatic;
+    AudioDownmix = HDFF_AUDIO_DOWNMIX_AUTOMATIC;
     OsdSize = 0;
     CecEnabled = 1;
     RemoteProtocol = 1;
@@ -35,18 +36,19 @@ cHdffSetup::cHdffSetup(void)
 
 bool cHdffSetup::SetupParse(const char *Name, const char *Value)
 {
-    if      (strcmp(Name, "Resolution")      == 0) Resolution      = atoi(Value);
-    else if (strcmp(Name, "TvFormat")        == 0) TvFormat        = atoi(Value);
-    else if (strcmp(Name, "VideoConversion") == 0) VideoConversion = atoi(Value);
-    else if (strcmp(Name, "AnalogueVideo")   == 0) AnalogueVideo   = atoi(Value);
-    else if (strcmp(Name, "AudioDelay")      == 0) AudioDelay      = atoi(Value);
-    else if (strcmp(Name, "AudioDownmix")    == 0) AudioDownmix    = atoi(Value);
-    else if (strcmp(Name, "OsdSize")         == 0) OsdSize         = atoi(Value);
-    else if (strcmp(Name, "CecEnabled")      == 0) CecEnabled      = atoi(Value);
-    else if (strcmp(Name, "RemoteProtocol")  == 0) RemoteProtocol  = atoi(Value);
-    else if (strcmp(Name, "RemoteAddress")   == 0) RemoteAddress   = atoi(Value);
-    else if (strcmp(Name, "HighLevelOsd")    == 0) HighLevelOsd    = atoi(Value);
-    else if (strcmp(Name, "TrueColorOsd")    == 0) TrueColorOsd    = atoi(Value);
+    if      (strcmp(Name, "Resolution")        == 0) Resolution        = atoi(Value);
+    else if (strcmp(Name, "VideoModeAdaption") == 0) VideoModeAdaption = atoi(Value);
+    else if (strcmp(Name, "TvFormat")          == 0) TvFormat          = atoi(Value);
+    else if (strcmp(Name, "VideoConversion")   == 0) VideoConversion   = atoi(Value);
+    else if (strcmp(Name, "AnalogueVideo")     == 0) AnalogueVideo     = atoi(Value);
+    else if (strcmp(Name, "AudioDelay")        == 0) AudioDelay        = atoi(Value);
+    else if (strcmp(Name, "AudioDownmix")      == 0) AudioDownmix      = atoi(Value);
+    else if (strcmp(Name, "OsdSize")           == 0) OsdSize           = atoi(Value);
+    else if (strcmp(Name, "CecEnabled")        == 0) CecEnabled        = atoi(Value);
+    else if (strcmp(Name, "RemoteProtocol")    == 0) RemoteProtocol    = atoi(Value);
+    else if (strcmp(Name, "RemoteAddress")     == 0) RemoteAddress     = atoi(Value);
+    else if (strcmp(Name, "HighLevelOsd")      == 0) HighLevelOsd      = atoi(Value);
+    else if (strcmp(Name, "TrueColorOsd")      == 0) TrueColorOsd      = atoi(Value);
     else return false;
     return true;
 }
@@ -66,7 +68,7 @@ void cHdffSetup::GetOsdSize(int &Width, int &Height, double &PixelAspect)
             Width = 720;
             Height = 576;
         }
-        if (TvFormat == HDFF::tvFormat16by9)
+        if (TvFormat == HDFF_TV_FORMAT_16_BY_9)
             PixelAspect = 16.0 / 9.0;
         else
             PixelAspect = 4.0 / 3.0;
@@ -94,25 +96,26 @@ void cHdffSetup::GetOsdSize(int &Width, int &Height, double &PixelAspect)
     PixelAspect /= double(Width) / Height;
 }
 
-HDFF::eHdmiVideoMode cHdffSetup::GetVideoMode(void)
+HdffVideoMode_t cHdffSetup::GetVideoMode(void)
 {
     switch (Resolution)
     {
         case kResolution1080i:
         default:
-            return HDFF::videoMode1080i50;
+            return HDFF_VIDEO_MODE_1080I50;
         case kResolution720p:
-            return HDFF::videoMode720p50;
+            return HDFF_VIDEO_MODE_720P50;
         case kResolution576p:
-            return HDFF::videoMode576p50;
+            return HDFF_VIDEO_MODE_576P50;
         case kResolution576i:
-            return HDFF::videoMode576i50;
+            return HDFF_VIDEO_MODE_576I50;
     }
 }
 
 cHdffSetupPage::cHdffSetupPage(HDFF::cHdffCmdIf * pHdffCmdIf)
 {
     const int kResolutions = 4;
+    const int kVideoModeAdaptions = 4;
     const int kTvFormats = 2;
     const int kVideoConversions = 6;
     const int kAnalogueVideos = 4;
@@ -128,6 +131,14 @@ cHdffSetupPage::cHdffSetupPage(HDFF::cHdffCmdIf * pHdffCmdIf)
         "576i",
     };
 
+    static const char * VideoModeAdaptionItems[kVideoModeAdaptions] =
+    {
+        tr("Off"),
+        tr("Frame rate"),
+        tr("HD Only"),
+        tr("Always")
+    };
+
     static const char * TvFormatItems[kTvFormats] =
     {
         "4/3",
@@ -182,6 +193,7 @@ cHdffSetupPage::cHdffSetupPage(HDFF::cHdffCmdIf * pHdffCmdIf)
     mNewHdffSetup = gHdffSetup;
 
     Add(new cMenuEditStraItem(tr("Resolution"), &mNewHdffSetup.Resolution, kResolutions, ResolutionItems));
+    Add(new cMenuEditStraItem(tr("Video Mode Adaption"), &mNewHdffSetup.VideoModeAdaption, kVideoModeAdaptions, VideoModeAdaptionItems));
     Add(new cMenuEditStraItem(tr("TV format"), &mNewHdffSetup.TvFormat, kTvFormats, TvFormatItems));
     Add(new cMenuEditStraItem(tr("Video Conversion"), &mNewHdffSetup.VideoConversion, kVideoConversions, VideoConversionItems));
     Add(new cMenuEditStraItem(tr("Analogue Video"), &mNewHdffSetup.AnalogueVideo, kAnalogueVideos, AnalogueVideoItems));
@@ -202,6 +214,7 @@ cHdffSetupPage::~cHdffSetupPage(void)
 void cHdffSetupPage::Store(void)
 {
     SetupStore("Resolution", mNewHdffSetup.Resolution);
+    SetupStore("VideoModeAdaption", mNewHdffSetup.VideoModeAdaption);
     SetupStore("TvFormat", mNewHdffSetup.TvFormat);
     SetupStore("VideoConversion", mNewHdffSetup.VideoConversion);
     SetupStore("AnalogueVideo", mNewHdffSetup.AnalogueVideo);
@@ -220,26 +233,27 @@ void cHdffSetupPage::Store(void)
         {
             mHdffCmdIf->CmdHdmiSetVideoMode(mNewHdffSetup.GetVideoMode());
         }
-        HDFF::tVideoFormat videoFormat;
-        HDFF::tHdmiConfig hdmiConfig;
+        HdffVideoFormat_t videoFormat;
+        HdffHdmiConfig_t hdmiConfig;
 
         videoFormat.AutomaticEnabled = true;
         videoFormat.AfdEnabled = true;
-        videoFormat.TvFormat = (HDFF::eTvFormat) mNewHdffSetup.TvFormat;
-        videoFormat.VideoConversion = (HDFF::eVideoConversion) mNewHdffSetup.VideoConversion;
+        videoFormat.TvFormat = (HdffTvFormat_t) mNewHdffSetup.TvFormat;
+        videoFormat.VideoConversion = (HdffVideoConversion_t) mNewHdffSetup.VideoConversion;
         mHdffCmdIf->CmdAvSetVideoFormat(0, &videoFormat);
 
         mHdffCmdIf->CmdAvSetAudioDelay(mNewHdffSetup.AudioDelay);
-        mHdffCmdIf->CmdAvSetAudioDownmix((HDFF::eDownmixMode) mNewHdffSetup.AudioDownmix);
+        mHdffCmdIf->CmdAvSetAudioDownmix((HdffAudioDownmixMode_t) mNewHdffSetup.AudioDownmix);
 
-        mHdffCmdIf->CmdMuxSetVideoOut((HDFF::eVideoOut) mNewHdffSetup.AnalogueVideo);
+        mHdffCmdIf->CmdMuxSetVideoOut((HdffVideoOut_t) mNewHdffSetup.AnalogueVideo);
 
         hdmiConfig.TransmitAudio = true;
         hdmiConfig.ForceDviMode = false;
         hdmiConfig.CecEnabled = mNewHdffSetup.CecEnabled;
+        hdmiConfig.VideoModeAdaption = (HdffVideoModeAdaption_t) mNewHdffSetup.VideoModeAdaption;
         mHdffCmdIf->CmdHdmiConfigure(&hdmiConfig);
 
-        mHdffCmdIf->CmdRemoteSetProtocol((HDFF::eRemoteProtocol) mNewHdffSetup.RemoteProtocol);
+        mHdffCmdIf->CmdRemoteSetProtocol((HdffRemoteProtocol_t) mNewHdffSetup.RemoteProtocol);
         mHdffCmdIf->CmdRemoteSetAddressFilter(mNewHdffSetup.RemoteAddress >= 0, mNewHdffSetup.RemoteAddress);
     }
 
