@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: setup.h 1.8 2011/04/17 11:20:22 kls Exp $
+ * $Id: setup.h 1.10 2011/12/04 15:32:13 kls Exp $
  */
 
 #ifndef _HDFF_SETUP_H_
@@ -17,9 +17,10 @@ struct cHdffSetup
     cHdffSetup(void);
     bool SetupParse(const char * Name, const char * Value);
     void GetOsdSize(int &Width, int &Height, double &PixelAspect);
-    HDFF::eHdmiVideoMode GetVideoMode(void);
+    HdffVideoMode_t GetVideoMode(void);
 
     int Resolution;
+    int VideoModeAdaption;
     int TvFormat;
     int VideoConversion;
     int AnalogueVideo;
