@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: descriptor.h 2.0 2007/02/03 11:45:58 kls Exp $
+ *   $Id: descriptor.h 2.2 2011/06/15 21:26:00 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -361,6 +361,31 @@ protected:
    virtual void Parse();
 };
 
+class ContentIdentifierDescriptor : public Descriptor {
+public:
+   class Identifier : public LoopElement {
+   public:
+      String identifier;
+      int getCridType() const;
+      int getCridLocation() const;
+      virtual int getLength() { return sizeof(content_identifier_entry)+identifier.getLength(); }
+   protected:
+      virtual void Parse();
+   private:
+      const content_identifier_entry *s;
+   };
+   StructureLoop<Identifier> identifierLoop;
+protected:
+   virtual void Parse();
+};
+
+class DefaultAuthorityDescriptor : public Descriptor {
+public:
+   String DefaultAuthority; //ID
+protected:
+   virtual void Parse();
+};
+
 //abstract base class
 class MultilingualNameDescriptor : public Descriptor {
 public:
@@ -660,6 +685,16 @@ private:
    const descr_application_icons_descriptor_end *s;
 };
 
+class RegistrationDescriptor : public Descriptor {
+public:
+   int getFormatIdentifier() const;
+   CharArray privateData;
+protected:
+   virtual void Parse();
+private:
+   const descr_registration *s;
+};
+
 } //end of namespace
 
 #endif //LIBSI_TABLE_H
