@@ -10,7 +10,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: headers.h 2.0 2007/02/03 11:45:58 kls Exp $
+ *   $Id: headers.h 2.2 2011/06/15 21:26:00 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -1680,6 +1680,24 @@ struct descr_content_identifier {
    u_char descriptor_length                      :8;
 };
 
+struct content_identifier_entry {
+#if BYTE_ORDER == BIG_ENDIAN
+   u_char crid_type                              :6;
+   u_char crid_location                          :2;
+#else
+   u_char crid_location                          :2;
+   u_char crid_type                              :6;
+#endif
+  union {
+    u_char crid_length                           :8;
+    u_char crid_ref_hi                           :8;
+  };
+  union {
+    u_char crid_byte                             :8;
+    u_char crid_ref_lo                           :8;
+  };
+};
+
 /* 0x77 time_slice_fec_identifier_descriptor (ETSI EN 301 192) */
 
 struct descr_time_slice_fec_identifier {
@@ -2006,6 +2024,18 @@ struct item_premiere_content_transmission_time {
    u_char start_time_s                           :8;
 };
 
+/* 0x05 registration_descriptor */
+
+#define DESCR_REGISTRATION_LEN 6
+struct descr_registration {
+   u_char descriptor_tag                         :8;
+   u_char descriptor_length                      :8;
+   u_char format_identifier_hi_hi                :8;
+   u_char format_identifier_hi_lo                :8;
+   u_char format_identifier_lo_hi                :8;
+   u_char format_identifier_lo_lo                :8;
+};
+
 } //end of namespace
 
 #endif //LIBSI_HEADERS_H
