@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.c 2.21 2011/04/17 14:25:07 kls Exp $
+ * $Id: osd.c 2.23 2011/08/15 09:27:39 kls Exp $
  */
 
 #include "osd.h"
@@ -594,9 +594,10 @@ void cBitmap::DrawRectangle(int x1, int y1, int x2, int y2, tColor Color)
      x2 = min(x2, width - 1);
      y2 = min(y2, height - 1);
      tIndex c = Index(Color);
-     for (int y = y1; y <= y2; y++)
+     for (int y = y1; y <= y2; y++) {
          for (int x = x1; x <= x2; x++)
              SetIndex(x, y, c);
+         }
      }
 }
 
@@ -1599,9 +1600,11 @@ int cOsd::osdTop = 0;
 int cOsd::osdWidth = 0;
 int cOsd::osdHeight = 0;
 cVector<cOsd *> cOsd::Osds;
+cMutex cOsd::mutex;
 
 cOsd::cOsd(int Left, int Top, uint Level)
 {
+  cMutexLock MutexLock(&mutex);
   isTrueColor = false;
   savedBitmap = NULL;
   numBitmaps = 0;
@@ -1623,6 +1626,7 @@ cOsd::cOsd(int Left, int Top, uint Level)
 
 cOsd::~cOsd()
 {
+  cMutexLock MutexLock(&mutex);
   for (int i = 0; i < numBitmaps; i++)
       delete bitmaps[i];
   delete savedBitmap;
@@ -1943,6 +1947,7 @@ cOsdProvider::~cOsdProvider()
 
 cOsd *cOsdProvider::NewOsd(int Left, int Top, uint Level)
 {
+  cMutexLock MutexLock(&cOsd::mutex);
   if (Level == OSD_LEVEL_DEFAULT && cOsd::IsOpen())
      esyslog("ERROR: attempt to open OSD while it is already open - using dummy OSD!");
   else if (osdProvider) {
