@@ -15,10 +15,6 @@
 #include "skins.h"
 #include "tools.h"
 
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-#include "submenu.h"
-#endif /* SETUP & PINPLUGIN */
-
 enum eOSState { osUnknown,
                 osContinue,
                 osSchedule,
@@ -55,26 +51,16 @@ private:
   char *text;
   eOSState state;
   bool selectable;
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-  cSubMenuNode* subMenu;
-#endif /* SETUP & PINPLUGIN */
 protected:
   bool fresh;
 public:
   cOsdItem(eOSState State = osUnknown);
   cOsdItem(const char *Text, eOSState State = osUnknown, bool Selectable = true);
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-  cOsdItem(const char *Text, eOSState State, cSubMenuNode* SubMenu);
-#endif /* SETUP & PINPLUGIN */
   virtual ~cOsdItem();
   bool Selectable(void) const { return selectable; }
   void SetText(const char *Text, bool Copy = true);
   void SetSelectable(bool Selectable);
   void SetFresh(bool Fresh);
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-  void SetSubMenu(cSubMenuNode* SubMenu) { subMenu = SubMenu; }
-  cSubMenuNode* SubMenu() { return subMenu; }
-#endif /* SETUP & PINPLUGIN */
   const char *Text(void) const { return text; }
   virtual void Set(void) {}
   virtual eOSState ProcessKey(eKeys Key);
@@ -109,10 +95,6 @@ private:
   char *status;
   int digit;
   bool hasHotkeys;
-#ifdef USE_LIEMIEXT
-  int key_nr;
-  cTimeMs lastActivity;
-#endif /* LIEMIEXT */
 protected:
   void SetDisplayMenu(void);
   cSkinDisplayMenu *DisplayMenu(void) { return displayMenu; }
@@ -149,9 +131,6 @@ public:
   void Ins(cOsdItem *Item, bool Current = false, cOsdItem *Before = NULL);
   virtual void Display(void);
   virtual eOSState ProcessKey(eKeys Key);
-#ifdef USE_GRAPHTFT
-  virtual const char* MenuKind() { return "MenuUnknown"; }
-#endif /* GRAPHTFT */
   };
 
 #endif //__OSDBASE_H
