@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 2.8 2010/01/17 12:23:31 kls Exp $
+ * $Id: svdrp.c 2.12 2011/12/04 13:58:33 kls Exp $
  */
 
 #include "svdrp.h"
@@ -219,10 +219,12 @@ const char *HelpPages[] = {
   "    image format defaults to JPEG.",
   "HELP [ <topic> ]\n"
   "    The HELP command gives help info.",
-  "HITK [ <key> ]\n"
+  "HITK [ <key> ... ]\n"
   "    Hit the given remote control key. Without option a list of all\n"
-  "    valid key names is given.",
-  "LSTC [ :groups | <number> | <name> ]\n"
+  "    valid key names is given. If more than one key is given, they are\n"
+  "    entered into the remote control queue in the given sequence. There\n"
+  "    can be up to 31 keys.",
+  "LSTC [ :groups | <number> | <name> | <id> ]\n"
   "    List channels. Without option, all channels are listed. Otherwise\n"
   "    only the given channel is listed. If a name is given, all channels\n"
   "    containing the given string as part of their name are listed.\n"
@@ -304,10 +306,6 @@ const char *HelpPages[] = {
   "REMO [ on | off ]\n"
   "    Turns the remote control on or off. Without a parameter, the current\n"
   "    status of the remote control is reported.",
-#ifdef USE_LIEMIEXT
-  "RENR <number> <new name>\n"
-  "    Rename recording. Number must be the Number as returned by LSTR command.",
-#endif /* LIEMIEXT */
   "SCAN\n"
   "    Forces an EPG scan. If this is a single DVB device system, the scan\n"
   "    will be done on the primary device unless it is currently recording.",
@@ -317,6 +315,9 @@ const char *HelpPages[] = {
   "    Updates a timer. Settings must be in the same format as returned\n"
   "    by the LSTT command. If a timer with the same channel, day, start\n"
   "    and stop time does not yet exists, it will be created.",
+  "UPDR\n"
+  "    Initiates a re-read of the recordings directory, which is the SVDRP\n"
+  "    equivalent to 'touch .update'.",
   "VOLU [ <number> | + | - | mute ]\n"
   "    Set the audio volume to the given number (which is limited to the range\n"
   "    0...255). If the special options '+' or '-' are given, the volume will\n"
@@ -906,13 +907,28 @@ void cSVDRP::CmdHELP(const char *Option)
 void cSVDRP::CmdHITK(const char *Option)
 {
   if (*Option) {
-     eKeys k = cKey::FromString(Option);
-     if (k != kNone) {
-        cRemote::Put(k);
-        Reply(250, "Key \"%s\" accepted", Option);
-        }
-     else
-        Reply(504, "Unknown key: \"%s\"", Option);
+     char buf[strlen(Option) + 1];
+     strcpy(buf, Option);
+     const char *delim = " \t";
+     char *strtok_next;
+     char *p = strtok_r(buf, delim, &strtok_next);
+     int NumKeys = 0;
+     while (p) {
+           eKeys k = cKey::FromString(p);
+           if (k != kNone) {
+              if (!cRemote::Put(k)) {
+                 Reply(451, "Too many keys in \"%s\" (only %d accepted)", Option, NumKeys);
+                 return;
+                 }
+              }
+           else {
+              Reply(504, "Unknown key: \"%s\"", p);
+              return;
+              }
+           NumKeys++;
+           p = strtok_r(NULL, delim, &strtok_next);
+           }
+     Reply(250, "Key%s \"%s\" accepted", NumKeys > 1 ? "s" : "", Option);
      }
   else {
      Reply(-214, "Valid <key> names for the HITK command:");
@@ -935,16 +951,18 @@ void cSVDRP::CmdLSTC(const char *Option)
            Reply(501, "Channel \"%s\" not defined", Option);
         }
      else {
-        cChannel *next = NULL;
-        for (cChannel *channel = Channels.First(); channel; channel = Channels.Next(channel)) {
-            if (!channel->GroupSep()) {
-               if (strcasestr(channel->Name(), Option)) {
-                  if (next)
-                     Reply(-250, "%d %s", next->Number(), *next->ToText());
-                  next = channel;
-                  }
-               }
-            }
+        cChannel *next = Channels.GetByChannelID(tChannelID::FromString(Option));
+        if (!next) {
+           for (cChannel *channel = Channels.First(); channel; channel = Channels.Next(channel)) {
+              if (!channel->GroupSep()) {
+                 if (strcasestr(channel->Name(), Option)) {
+                    if (next)
+                       Reply(-250, "%d %s", next->Number(), *next->ToText());
+                    next = channel;
+                    }
+                 }
+              }
+           }
         if (next)
            Reply(250, "%d %s", next->Number(), *next->ToText());
         else
@@ -1497,38 +1515,6 @@ void cSVDRP::CmdSCAN(const char *Option)
   Reply(250, "EPG scan triggered");
 }
 
-#ifdef USE_LIEMIEXT
-void cSVDRP::CmdRENR(const char *Option)
-{
-  bool recordings = Recordings.Update(true);
-  if (recordings) {
-     if (*Option) {
-        char *tail;
-        int n = strtol(Option, &tail, 10);
-        cRecording *recording = Recordings.Get(n - 1);
-        if (recording && tail && tail != Option) {
-           char *oldName = strdup(recording->Name());
-           tail = skipspace(tail);
-           if (recording->Rename(tail)) {
-              Reply(250, "Renamed \"%s\" to \"%s\"", oldName, recording->Name());
-              Recordings.ChangeState();
-              Recordings.TouchUpdate();
-              }
-           else
-              Reply(501, "Renaming \"%s\" to \"%s\" failed", oldName, tail);
-           free(oldName);
-           }
-        else
-          Reply(501, "Recording not found or wrong syntax");
-        }
-     else
-        Reply(501, "Missing Input settings");
-     }
-  else
-     Reply(550, "No recordings available");
-}
-#endif /* LIEMIEXT */
-
 void cSVDRP::CmdSTAT(const char *Option)
 {
   if (*Option) {
@@ -1576,6 +1562,12 @@ void cSVDRP::CmdUPDT(const char *Option)
      Reply(501, "Missing timer settings");
 }
 
+void cSVDRP::CmdUPDR(const char *Option)
+{
+  Recordings.Update(false);
+  Reply(250, "Re-read of recordings directory triggered");
+}
+
 void cSVDRP::CmdVOLU(const char *Option)
 {
   if (*Option) {
@@ -1644,11 +1636,9 @@ void cSVDRP::Execute(char *Cmd)
   else if (CMD("PLUG"))  CmdPLUG(s);
   else if (CMD("PUTE"))  CmdPUTE(s);
   else if (CMD("REMO"))  CmdREMO(s);
-#ifdef USE_LIEMIEXT
-  else if (CMD("RENR"))  CmdRENR(s);
-#endif /* LIEMIEXT */
   else if (CMD("SCAN"))  CmdSCAN(s);
   else if (CMD("STAT"))  CmdSTAT(s);
+  else if (CMD("UPDR"))  CmdUPDR(s);
   else if (CMD("UPDT"))  CmdUPDT(s);
   else if (CMD("VOLU"))  CmdVOLU(s);
   else if (CMD("QUIT"))  Close(true);
@@ -1703,8 +1693,16 @@ bool cSVDRP::Process(void)
                  }
               else {
                  if (numChars >= length - 1) {
-                    length += BUFSIZ;
-                    cmdLine = (char *)realloc(cmdLine, length);
+                    int NewLength = length + BUFSIZ;
+                    if (char *NewBuffer = (char *)realloc(cmdLine, NewLength)) {
+                       length = NewLength;
+                       cmdLine = NewBuffer;
+                       }
+                    else {
+                       esyslog("ERROR: out of memory");
+                       Close();
+                       break;
+                       }
                     }
                  cmdLine[numChars++] = c;
                  cmdLine[numChars] = 0;
